@@ -20,6 +20,7 @@
 
 #include <glib-object.h>
 #include "spice-types.h"
+#include "spice-uri.h"
 #include "spice-glib-enums.h"
 #include "spice-util.h"
 
@@ -92,6 +93,7 @@ void spice_session_disconnect(SpiceSession *session);
 GList *spice_session_get_channels(SpiceSession *session);
 gboolean spice_session_has_channel_type(SpiceSession *session, gint type);
 gboolean spice_session_get_read_only(SpiceSession *session);
+SpiceURI *spice_session_get_proxy_uri(SpiceSession *session);
 
 G_END_DECLS
 
