@@ -1208,6 +1208,7 @@ static void recent_add(SpiceSession *session)
 static void main_channel_event(SpiceChannel *channel, SpiceChannelEvent event,
                                gpointer data)
 {
+    const GError *error = NULL;
     spice_connection *conn = data;
     char password[64];
     int rc;
@@ -1231,7 +1232,12 @@ static void main_channel_event(SpiceChannel *channel, SpiceChannelEvent event,
     case SPICE_CHANNEL_ERROR_TLS:
     case SPICE_CHANNEL_ERROR_LINK:
     case SPICE_CHANNEL_ERROR_CONNECT:
+        error = spice_channel_get_error(channel);
         g_message("main channel: failed to connect");
+        if (error) {
+            g_message("channel error: %s", error->message);
+        }
+
         rc = connect_dialog(conn->session);
         if (rc == 0) {
             connection_connect(conn);
@@ -1475,24 +1481,18 @@ static void port_opened(SpiceChannel *channel, GParamSpec *pspec,
         /* only send a break event and disconnect */
         if (g_strcmp0(name, "org.spice.spicy.break") == 0) {
             spice_port_event(port, SPICE_PORT_EVENT_BREAK);
+            spice_channel_flush_async(channel, NULL, port_flushed_cb, conn);
         }
 
         /* handle the first spicy port and connect it to stdin/out */
         if (g_strcmp0(name, "org.spice.spicy") == 0 && stdin_port == NULL) {
             stdin_port = port;
-            goto end;
         }
-
-        if (port == stdin_port)
-            goto end;
-
-        spice_channel_flush_async(channel, NULL, port_flushed_cb, conn);
     } else {
         if (port == stdin_port)
             stdin_port = NULL;
     }
 
-end:
     g_free(name);
 }
 
